from app.utils import background_delete_job
from flask import (abort, current_app, flash, redirect, request,
                   render_template, url_for, send_from_directory)
from flask_login import current_user, login_required
from . import main
from .forms import AddCorpusFileForm, CreateCorpusForm, QueryForm, QueryDownloadForm
from .. import db
from ..models import Corpus, CorpusFile, Job, JobInput, JobResult
from werkzeug.utils import secure_filename
import os
import threading
import logging


@main.route('/')
def index():
    return render_template('main/index.html.j2', title='Opaque')


@main.route('/corpora/new', methods=['POST'])
@login_required
def corpus_new():
    create_corpus_form = CreateCorpusForm()
    if create_corpus_form.validate_on_submit():
        corpus = Corpus(creator=current_user,
                        description=create_corpus_form.description.data,
                        title=create_corpus_form.title.data)
        db.session.add(corpus)
        db.session.commit()
        dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                           str(corpus.user_id),
                           'corpora',
                           str(corpus.id))
        try:
            os.makedirs(dir)
        except OSError:
            flash('OSError!')
            db.session.remove(corpus)
            db.session.commit()
        flash('Corpus created!')
        return redirect(url_for('main.corpus', corpus_id=corpus.id))


@main.route('/corpora/<int:corpus_id>')
@login_required
def corpus(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    return render_template('main/corpora/corpus.html.j2',
                           add_corpus_file_form=AddCorpusFileForm(),
                           corpus=corpus,
                           title='Corpus')


@main.route('/corpora/<int:corpus_id>/analysis', methods=['GET', 'POST'])
@login_required
def corpus_analysis(corpus_id):
    logger = logging.getLogger(__name__)
    corpus = Corpus.query.get_or_404(corpus_id)
    query = request.args.get('query')
    logger.warning('Query first: {}'.format(query))
    hits_per_page = request.args.get('hits_per_page', 30)
    context = request.args.get('context', 10)
    dl_form = QueryDownloadForm()
    form = QueryForm(hits_per_page=hits_per_page, context=context, query=query)
    if form.validate_on_submit():
        logger = logging.getLogger(__name__)
        logger.warning('Data has been sent!')
        logger.warning('Data labels: {data}'.format(data=[data for data in form.data]))
        logger.warning('Query Second: {q}'.format(q=form.query.data))
        logger.warning('Hits: {hits}'.format(hits=form.hits_per_page.data))
        logger.warning('Context: {context}'.format(context=form.context.data))
        flash('Query has been sent!')
        query = form.query.data
        hits_per_page = form.hits_per_page.data
        context = form.context.data
        logger.warning('Query Thrid: {sq}'.format(sq=query))
        return redirect(url_for('main.corpus_analysis',
                                corpus_id=corpus_id,
                                query=query,
                                hits_per_page=hits_per_page,
                                context=context))
    return render_template('main/corpora/corpus_analysis.html.j2',
                           corpus=corpus,
                           form=form,
                           dl_form=dl_form,
                           title='Corpus: ' + corpus.title)


@main.route('/corpora/<int:corpus_id>/delete')
@login_required
def corpus_delete(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    delete_thread = threading.Thread(corpus.delete())
    delete_thread.start()
    flash('Corpus has been deleted!')
    return redirect(url_for('main.dashboard'))


@main.route('/corpora/<int:corpus_id>/files/new', methods=['POST'])
@login_required
def corpus_file_new(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    add_corpus_file_form = AddCorpusFileForm()
    if not add_corpus_file_form.validate_on_submit():
        abort(400)
    file = add_corpus_file_form.file.data
    filename = secure_filename(file.filename)
    for corpus_file in corpus.files:
        if filename == corpus_file.filename:
            flash('File already registered to this corpus.')
            return redirect(url_for('main.corpus', corpus_id=corpus_id))
    # Save the file
    dir = os.path.join(str(corpus.user_id), 'corpora', str(corpus.id))
    file_path = os.path.join(
        current_app.config['OPAQUE_STORAGE_DIRECTORY'], dir, filename
    )
    file.save(file_path)
    # Gather information to create new corpus file database entry
    author = add_corpus_file_form.author.data
    publishing_year = add_corpus_file_form.publishing_year.data
    title = add_corpus_file_form.title.data
    corpus_file = CorpusFile(author=author,
                             corpus=corpus,
                             dir=dir,
                             filename=filename,
                             publishing_year=publishing_year,
                             title=title)
    db.session.add(corpus_file)
    db.session.commit()
    flash('Corpus file added!')
    return redirect(url_for('main.corpus', corpus_id=corpus_id))


@main.route('/corpora/<int:corpus_id>/files/<int:corpus_file_id>/delete')
@login_required
def corpus_file_delete(corpus_id, corpus_file_id):
    corpus_file = CorpusFile.query.get_or_404(corpus_file_id)
    if not corpus_file.corpus_id == corpus_id:
        abort(404)
    if not (corpus_file.corpus.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    delete_thread = threading.Thread(corpus_file.delete())
    delete_thread.start()
    flash('Corpus file deleted!')
    return redirect(url_for('main.corpus', corpus_id=corpus_id))


@main.route('/corpora/<int:corpus_id>/files/<int:corpus_file_id>/download')
@login_required
def corpus_file_download(corpus_id, corpus_file_id):
    corpus_file = CorpusFile.query.get_or_404(corpus_file_id)
    if not corpus_file.corpus_id == corpus_id:
        abort(404)
    if not (corpus_file.corpus.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       corpus_file.dir)
    return send_from_directory(as_attachment=True,
                               directory=dir,
                               filename=corpus_file.filename)


@main.route('/dashboard', methods=['GET', 'POST'])
@login_required
def dashboard():
    create_corpus_form = CreateCorpusForm()
    if create_corpus_form.validate_on_submit():
        corpus = Corpus(creator=current_user._get_current_object(),
                        description=create_corpus_form.description.data,
                        title=create_corpus_form.title.data)
        db.session.add(corpus)
        db.session.commit()
        dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                           str(corpus.user_id),
                           'corpora',
                           str(corpus.id))
        try:
            os.makedirs(dir)
        except OSError:
            flash('OSError!')
            db.session.remove(corpus)
            db.session.commit()
        flash('Corpus created!')
        return redirect(url_for('main.dashboard'))
    return render_template('main/dashboard.html.j2',
                           create_corpus_form=create_corpus_form,
                           title='Dashboard')


@main.route('/jobs/<int:job_id>')
@login_required
def job(job_id):
    job = Job.query.get_or_404(job_id)
    if not (job.creator == current_user or current_user.is_administrator()):
        abort(403)
    return render_template('main/jobs/job.html.j2', job=job, title='Job')


@main.route('/jobs/<int:job_id>/delete')
@login_required
def delete_job(job_id):
    delete_thread = threading.Thread(
        target=background_delete_job,
        args=(current_app._get_current_object(), job_id)
    )
    delete_thread.start()
    flash('Job has been deleted!')
    return redirect(url_for('main.dashboard'))


@main.route('/jobs/<int:job_id>/inputs/<int:job_input_id>/download')
@login_required
def job_input_download(job_id, job_input_id):
    job_input = JobInput.query.get_or_404(job_input_id)
    if not job_input.job_id == job_id:
        abort(404)
    if not (job_input.job.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       job_input.dir)
    return send_from_directory(as_attachment=True,
                               directory=dir,
                               filename=job_input.filename)


@main.route('/jobs/<int:job_id>/results/<int:job_result_id>/download')
@login_required
def job_result_download(job_id, job_result_id):
    job_result = JobResult.query.get_or_404(job_result_id)
    if not job_result.job_id == job_id:
        abort(404)
    if not (job_result.job.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       job_result.dir)
    return send_from_directory(as_attachment=True,
                               directory=dir,
                               filename=job_result.filename)
